import { APIGatewayProxyEvent, Context } from "aws-lambda";
import { Request } from "./types";
import { Static, TSchema } from "@sinclair/typebox";
declare type AnyRequest<TBody> = {
    pathParams: any;
    queryParams: any;
    body: TBody;
};
export declare type RouteHandlerDefinition = {
    method: string;
    url: string;
    body?: TSchema;
    responses: Responses;
    handler: (req: AnyRequest<any>, apiParams: {
        originalEvent: APIGatewayProxyEvent;
        context: Context;
    }) => Promise<Response<any, any>>;
};
declare type HttpMethod<R, M extends HTTPMethod> = <A extends string, S extends NumericKeysOf<Resp>, B extends TSchema = never, Resp extends Responses = typeof DefaultResponses>(path: A, body?: M extends HTTPRead ? never : B, responses?: Resp) => (handler: (req: Request<A, Static<B>>, originalEvent: {
    originalEvent: APIGatewayProxyEvent;
    context: Context;
}) => Promise<Response<Resp, S>>) => Router<R>;
export declare type Router<R> = R & {
    get: HttpMethod<R, "get">;
    head: HttpMethod<R, "head">;
    options: HttpMethod<R, "options">;
    post: HttpMethod<R, "post">;
    put: HttpMethod<R, "put">;
    delete: HttpMethod<R, "delete">;
};
export declare type RouteHandlers = {
    handlers: readonly RouteHandlerDefinition[];
};
declare type HTTPRead = "get" | "options" | "head";
declare type HTTPWrite = "post" | "put" | "delete";
declare type HTTPMethod = HTTPRead | HTTPWrite;
declare type NumericKeysOf<T> = keyof {
    [I in keyof T]: T[I] extends number ? I : never;
};
declare type Responses = {
    [K: number]: TSchema;
};
declare type Response<R extends Responses, Status extends NumericKeysOf<R>> = {
    statusCode: Status;
    headers?: {
        [header: string]: boolean | number | string;
    };
    body: Static<R[Status]>;
};
declare const DefaultResponses: {
    200: import("@sinclair/typebox").TUnion<[import("@sinclair/typebox").TDict<any>, import("@sinclair/typebox").TString]>;
    201: import("@sinclair/typebox").TUnion<[import("@sinclair/typebox").TDict<any>, import("@sinclair/typebox").TString]>;
    400: import("@sinclair/typebox").TUnion<[import("@sinclair/typebox").TDict<any>, import("@sinclair/typebox").TString]>;
    404: import("@sinclair/typebox").TUnion<[import("@sinclair/typebox").TDict<any>, import("@sinclair/typebox").TString]>;
    500: import("@sinclair/typebox").TUnion<[import("@sinclair/typebox").TDict<any>, import("@sinclair/typebox").TString]>;
};
export declare const Router: (handlers?: readonly RouteHandlerDefinition[]) => Router<RouteHandlers>;
export {};
